import random, sys

class Person:
    def __init__(self, game, name, start_time, work_left, health_left, tags):
        self.game = game
        self.id = name
        self.start_time = start_time
        self.work_left = work_left
        self.health_left = health_left
        self.tags = tags
        self.time_outdoors = 0
        self.water_accepted = 0
        self.status = "waiting"

    def check_status(self):
        if self.health_left <= 0:
            self.status = "fainted"
        elif self.work_left <= 0:
            self.status = "indoors"

    def normal_turn(self, turn):
        if self.status == "waiting":
            if turn == self.start_time:
                self.status = "outdoors"
            return
        if self.status == "outdoors":
            self.health_left -= 1
            self.work_left -= 1
            self.time_outdoors += 1
            self.check_status()

    def talking_turn(self):
        if self.status == "outdoors":
            self.health_left -= 1
            self.time_outdoors += 1
            self.check_status()

    def drink_water(self):
        if self.status == "outdoors":
            self.health_left += self.game.water_health_amount
            self.water_accepted += 1

    def hurry_home(self):
        if self.status == "outdoors":
            self.work_left -= self.game.hurrying_work_amount

def dialog(intro, options):
    print intro
    def print_options():
        print "Please type the number for one of the following options"
        for i in range(len(options)):
            print "(", i + 1, ")", options[i]
        print
    while True:
        print_options()
        try:
            val = int(raw_input()) - 1
            print
            if 0 > val or val >= len(options):
                raise ValueError
            return val
        except (ValueError, TypeError):
            print "Invalid Input\n"
            continue

def print_person(person):
    print person.id, "has been outdoors for", person.time_outdoors, "hours"
    print person.id, "has accepted water from you", person.water_accepted, "times"
    print "The following is a list of", person.id + "'s", "attributes:", ", ".join(person.tags)
    print

def print_people(people_list, filter_rule=None):
    if filter_rule == None:
        filter_rule = lambda x: True
    for person in people_list:
        if filter_rule(person):
            print_person(person)

class Game:
    def __init__(self, game_filename, dialog_filename):
        self.game_filename = game_filename
        self.dialog_filename = dialog_filename
        self.dialog_setup()
        self.reset()

    def reset(self):
        self.turn = 0
        self.conversation_target = None
        self.conversation_dialog = None
        self.conversation_step = None
        self.conversation_outcome = None

        f = open(self.game_filename)
        lines = f.readlines() + [""]
        self.hurrying_work_amount = int(lines[0])
        self.water_health_amount = int(lines[1])
        self.people = []
        i = 0
        name = None
        start_time = None
        work_left = None
        health_left = None
        tags = None
        for line in lines[3:]:
            line = line.strip()
            if i == 0:
                name = line
            if i == 1:
                start_time = int(line)
            if i == 2:
                work_left = int(line)
            if i == 3:
                health_left = int(line)
            if i > 3:
                if i == 4:
                    tags = []
                if line != "":
                    tags.append(line)
                else:
                    self.people.append(Person(self, name, start_time, work_left, health_left, tags))
                    i = -1
            i = i + 1
        f.close()
        self.active_people = []

    def dialog_setup(self):
        f = open(self.dialog_filename)
        lines = [line.strip() for line in f.readlines()]
        script = {}
        tag = ""
        line_type = None
        while lines:
            line = lines.pop(0)
            while not line:
                if lines:
                    line = lines.pop(0)
                else:
                    line = "~stop"
            if line == "~stop":
                break
            if line == "~tag":
                line = lines.pop(0)
                tag = line
            elif line in ["~w?1", "~w?2", "~w!maybe", "~w!yes", "~w!no", "~h?1", "~h?2", "~h!maybe", "~h!yes", "~h!no"]:
                line_type = line[1:]
            else:
                key = (tag, line_type)
                if key not in script:
                    script[key] = []
                script[key].append(line)
        f.close()
        self.script = script
    
    def describe_world(self):
        print "The number of people who have gone indoors so far is", sum([1 for person in self.people if person.status == "indoors"])
        print "The number of people who have fainted so far is", sum([1 for person in self.people if person.status == "fainted"])
        print
        print "The following people are currently outdoors:"
        print
        print_people(self.people, lambda x : x.status == "outdoors")

    def choose_action(self):
        intro = "What do you do?"
        options = []
        for person in self.active_people:
            options.append("talk to " + person.id)
        options.append("nothing")
        options.append("view the description of the people again and then decide")
        while True:
            option = dialog(intro, options)
            if option != len(options) - 1:
                break
            self.describe_world()
        if option == len(options) - 2:
            return None
        else:
            return self.active_people[option]

    def choose_topic(self):
        intro = "What do you say?"
        options = []
        options.append("Offer some water.")
        options.append("Suggest going indoors.")
        options.append("Go back and choose someone else to talk to.")
        option = dialog(intro, options)
        if option == 0:
            return "water"
        if option == 1:
            return "hurry"
        return "back"

    def act(self):
        if not self.conversation_target:
            loop = True
            while loop:
                target = self.choose_action()
                if not target:
                    print "You don't do anything\n"
                    return
                print "You chose to talk to", target.id
                print
                topic = self.choose_topic()
                if topic != "back":
                    loop = False
            self.generate_conversation(target, topic)
            self.next_lines(False)
        else:
            self.next_lines(True)

    def update_people(self):
        change = False
        for i in range(len(self.active_people) - 1, -1, -1):
            if self.active_people[i].status == "indoors":
                change = True
                print self.active_people[i].id, "went indoors"
                self.active_people.pop(i)
            elif self.active_people[i].status == "fainted":
                change = True
                print self.active_people[i].id, "fainted from the heat"
                self.active_people.pop(i)
        if change:
            print
            dialog("Continue?", ["ok"])
        self.active_people = [x for x in self.people if x.status == "outdoors"]

    def generate_conversation(self, person, topic):
        self.conversation_target = person
        self.conversation_step = 0
        
        if topic == "water":
            steps = random.randrange(5)
            good_outcome = random.choice([True, False, True])
            if person.health_left <= 2:
                steps = 0
                good_outcome = True
            if good_outcome:
                self.conversation_outcome = "water"
            else:
                self.conversation_outcome = "Nope"
            self.conversation_dialog = [[] for i in range(2 * (steps + 1))]
            for tag in person.tags + [""]:
                if (tag, "w?1") in self.script:
                    self.conversation_dialog[0].extend(self.script[(tag, "w?1")])
                for i in range(steps):
                    if (tag, "w?2") in self.script:
                        self.conversation_dialog[2*i+2].extend(self.script[(tag, "w?2")])
                    if (tag, "w!maybe") in self.script:
                        self.conversation_dialog[2*i+1].extend(self.script[(tag, "w!maybe")])
                if good_outcome:
                    final_type = "w!yes"
                else:
                    final_type = "w!no"
                if (tag, final_type) in self.script:
                    self.conversation_dialog[-1].extend(self.script[(tag, final_type)])
            self.conversation_dialog = [random.choice(x) for x in self.conversation_dialog]
        else:
            steps = random.randrange(1, 5)
            good_outcome = random.choice([False, False, True])
            if good_outcome:
                self.conversation_outcome = "hurry"
            else:
                self.conversation_outcome = "Nope"
            self.conversation_dialog = [[] for i in range(2 * (steps + 1))]
            for tag in person.tags + [""]:
                if (tag, "h?1") in self.script:
                    self.conversation_dialog[0].extend(self.script[(tag, "h?1")])
                for i in range(steps):
                    if (tag, "h?2") in self.script:
                        self.conversation_dialog[2*i+2].extend(self.script[(tag, "h?2")])
                    if (tag, "h!maybe") in self.script:
                        self.conversation_dialog[2*i+1].extend(self.script[(tag, "h!maybe")])
                if good_outcome:
                    final_type = "h!yes"
                else:
                    final_type = "h!no"
                if (tag, final_type) in self.script:
                    self.conversation_dialog[-1].extend(self.script[(tag, final_type)])
            self.conversation_dialog = [random.choice(x) for x in self.conversation_dialog]

    def next_lines(self, choice):
        if choice:
            intro = "You are talking to " + self.conversation_target.id + "\nWhat do you say?"
            options = []
            options.append("\"" + self.conversation_dialog[self.conversation_step] + "\"")
            options.append("\"Forget it. Bye.\"")
            if dialog(intro, options) == 1:
                self.conversation_target = None
                self.conversation_dialog = None
                self.conversation_step = None
                self.conversation_outcome = None
                return
        print "you say", "\"" + self.conversation_dialog[self.conversation_step] + "\""
        print self.conversation_target.id, "says",
        print "\"" + self.conversation_dialog[self.conversation_step + 1] + "\"\n"
        self.conversation_step += 2
        if self.conversation_step == len(self.conversation_dialog):
            if self.conversation_outcome == "water":
                self.conversation_target.drink_water()
                print self.conversation_target.id, "accepts some water"
            elif self.conversation_outcome == "hurry":
                self.conversation_target.hurry_home()
                print self.conversation_target.id, "listens to your advice"
            else:
                print self.conversation_target.id, "walks away from your conversation"
            print
            self.conversation_target = None
            self.conversation_dialog = None
            self.conversation_step = None
            self.conversation_outcome = None
        dialog("Continue?", ["ok"])

    def go(self):
        while True:
            for person in self.people:
                if person != self.conversation_target:
                    person.normal_turn(self.turn)
            if self.conversation_target:
                self.conversation_target.talking_turn()
            self.turn = self.turn + 1
            self.update_people()
            if not [1 for x in self.people if x.status in ["waiting", "outdoors"]]:
                break
            self.describe_world()
            self.act()
        a = sum([1 for person in self.people if person.status == "indoors"])
        b = len(self.people)
        self.reset()
        return (a,b)

    def play(self):
        while not dialog("Would you like to start?", ["Yep!", "quit"]):
            (a, b) = self.go()
            print a, "of", b, "people made it indoors\n"
            print "Consider trying again."
            print
            
if __name__ == "__main__":
    print "Please enter game filename\n"
    n1 = raw_input()
    print
    print "Please enter dialog filename\n"
    n2 = raw_input()
    print
    g = Game(n1, n2)
    g.play()
